from cozy.model.chapter import Chapter


class SingleFileChapter(Chapter):
    @property
    def name(self):
        pass

    @property
    def number(self):
        pass

    @property
    def disk(self):
        pass

    @property
    def position(self):
        pass

    @property
    def file(self):
        pass

    @property
    def length(self):
        pass

    @property
    def modified(self):
        pass
